@@ -9,8 +9,8 @@
 ;;             Bozhidar Batsov <bozhidar@batsov.dev>
 ;; URL: https://github.com/copilot-emacs/copilot.el
 ;; Package-Requires: ((emacs "27.2") (editorconfig "0.8.2") (jsonrpc "1.0.14") (f "0.20.0") (track-changes "1.4"))
-;; Package-Version: 20251117.510
-;; Package-Revision: 8e43edf1f3ef
+;; Package-Version: 20251210.1016
+;; Package-Revision: 7ee4758bb748
 ;; Keywords: convenience copilot
 
 ;; The MIT License (MIT)
@@ -123,6 +123,16 @@ performance."
   :type '(repeat function)
   :package-version '(copilot . "0.1"))
 
+(defconst copilot--hardcoded-clear-overlay-ignore-commands
+  '(universal-argument digit-argument negative-argument universal-argument-more)
+  "Hardcoded list of commands that should not clear the overlay.")
+
+(defcustom copilot-clear-overlay-on-commands '(beginning-of-visual-line)
+  "Commands that must clear the Copilot overlay before execution."
+  :group 'copilot
+  :type '(repeat function)
+  :package-version '(copilot . "0.4"))
+
 (defcustom copilot-indent-offset-warning-disable nil
   "Disable indentation warnings.
 
@@ -1036,8 +1046,53 @@ provided."
 
 (copilot--define-accept-completion-by-action copilot-accept-completion-by-word #'forward-word)
 (copilot--define-accept-completion-by-action copilot-accept-completion-by-line #'forward-line)
+(copilot--define-accept-completion-by-action copilot-accept-completion-by-sentence #'forward-sentence)
 (copilot--define-accept-completion-by-action copilot-accept-completion-by-paragraph #'forward-paragraph)
 
+(defun copilot--uppercase-char-p (char)
+  "Return non-nil when CHAR is uppercase in the current locale."
+  (and (characterp char)
+       (let ((up (upcase char))
+             (down (downcase char)))
+         (and (not (eq up down)) (eq char up)))))
+
+(defun copilot--completion-chunk-to-char (completion char count include-char)
+  "Return COMPLETION substring up to CHAR.
+COUNT specifies the occurrence.  INCLUDE-CHAR toggles whether CHAR stays
+in.  Uppercase CHAR disables `case-fold-search'."
+  (let ((count (or count 1)))
+    (when (<= count 0)
+      (user-error "COUNT must be positive"))
+    (with-temp-buffer
+      (insert completion)
+      (goto-char (point-min))
+      (let ((case-fold-search (if (copilot--uppercase-char-p char)
+                                  nil case-fold-search)))
+        (search-forward (char-to-string char) nil nil count)
+        (unless include-char
+          (backward-char))
+        (buffer-substring-no-properties (point-min) (point))))))
+
+(defun copilot-accept-completion-up-to-char (char &optional count)
+  "Accept completion up to but excluding CHAR.
+COUNT must be positive; signal an error if CHAR does not occur COUNT times.
+Uppercase CHAR disables `case-fold-search', mirroring `zap-up-to-char'."
+  (interactive (list (read-char "Accept completion up to char: ")
+                     (prefix-numeric-value current-prefix-arg)))
+  (copilot-accept-completion
+   (lambda (completion)
+     (copilot--completion-chunk-to-char completion char count nil))))
+
+(defun copilot-accept-completion-to-char (char &optional count)
+  "Accept completion up to and including CHAR.
+COUNT must be positive; signal an error if CHAR does not occur COUNT times.
+Uppercase CHAR disables `case-fold-search', mirroring `zap-to-char'."
+  (interactive (list (read-char "Accept completion through char: ")
+                     (prefix-numeric-value current-prefix-arg)))
+  (copilot-accept-completion
+   (lambda (completion)
+     (copilot--completion-chunk-to-char completion char count t))))
+
 (defun copilot--show-completion (completion-data)
   "Show COMPLETION-DATA."
   (when (copilot--satisfy-display-predicates)
@@ -1209,6 +1264,16 @@ Copilot will show completions only if all predicates return t."
                      (if (functionp pred) (funcall pred) nil))
                    ,disable)))
 
+(defun copilot--pre-command ()
+  "Handle `pre-command-hook' for Copilot.
+Clear the overlay for commands in `copilot-clear-overlay-on-commands' so
+that display-based motions (such as `beginning-of-visual-line') work
+consistently even when the overlay is visible."
+  (when (and this-command
+             (copilot--overlay-visible)
+             (memq this-command copilot-clear-overlay-on-commands))
+    (copilot-clear-overlay)))
+
 (defun copilot--satisfy-trigger-predicates ()
   "Return t if all trigger predicates are satisfied."
   (copilot--satisfy-predicates copilot-enable-predicates copilot-disable-predicates))
@@ -1224,6 +1289,10 @@ Copilot will show completions only if all predicates return t."
                        (or
                         (string-prefix-p "copilot-" (symbol-name this-command))
                         (member this-command copilot-clear-overlay-ignore-commands)
+                        ;; `this-original-command' captures remapped helpers like
+                        ;; `universal-argument-more' and `digit-argument'.
+                        (member this-original-command copilot-clear-overlay-ignore-commands)
+                        (member this-original-command copilot--hardcoded-clear-overlay-ignore-commands)
                         (copilot--self-insert this-command)))))
     (copilot-clear-overlay)
     (when copilot--post-command-timer
@@ -1287,6 +1356,7 @@ Use this for custom bindings in `copilot-mode'.")
 
 (defun copilot--mode-setup ()
   "Set up copilot mode."
+  (add-hook 'pre-command-hook #'copilot--pre-command nil 'local)
   (add-hook 'post-command-hook #'copilot--post-command nil 'local)
   ;; Hook onto both window-selection-change-functions and window-buffer-change-functions
   ;; since both are separate ways of 'focussing' a buffer.
@@ -1301,6 +1371,7 @@ Use this for custom bindings in `copilot-mode'.")
 
 (defun copilot--mode-teardown ()
   "Tear down copilot mode."
+  (remove-hook 'pre-command-hook #'copilot--pre-command 'local)
   (remove-hook 'post-command-hook #'copilot--post-command 'local)
   (remove-hook 'window-selection-change-functions #'copilot--on-doc-focus 'local)
   (remove-hook 'window-buffer-change-functions #'copilot--on-doc-focus 'local)
